pub struct CameraController {
base_path: String,
fullsize_map_image: RwLock<FullsizeMapImage>,
thumbnail_map_image: RwLock<ThumbnailMapImage>,
request_client: Arc<HTTPClient>,
}Expand description
A struct for managing camera-related operations and map snapshots.
Fields
base_path: StringThe base path for saving map image data.
fullsize_map_image: RwLock<FullsizeMapImage>The lock-protected full-size map image.
thumbnail_map_image: RwLock<ThumbnailMapImage>The lock-protected thumbnail map image.
request_client: Arc<HTTPClient>The HTTP client for sending requests.
Implementations
Sourceimpl CameraController
impl CameraController
Sourceconst LAST_IMG_END_DELAY: TimeDelta
const LAST_IMG_END_DELAY: TimeDelta
Constant minimum delay to perform another image.
Sourceconst ZO_IMG_FOLDER: &'static str = "zo_img/"
const ZO_IMG_FOLDER: &'static str = "zo_img/"
Directory where zoned objective images should be stored.
Sourceconst ZO_IMG_ACQ_DELAY: TimeDelta
const ZO_IMG_ACQ_DELAY: TimeDelta
Constant TimeDelta between images when in zoned objective acquisition.
Sourcepub fn start(base_path: String, request_client: Arc<HTTPClient>) -> Self
pub fn start(base_path: String, request_client: Arc<HTTPClient>) -> Self
Initializes the CameraController with the given base path and HTTP client.
Arguments
base_path- The base path for storing files.request_client- The HTTP client for sending requests.
Returns
A new instance of CameraController.
Sourcefn score_offset(
decoded_image: &RgbImage,
base: &FullsizeMapImage,
offset: Vec2D<u32>,
) -> Vec2D<i32>
fn score_offset( decoded_image: &RgbImage, base: &FullsizeMapImage, offset: Vec2D<u32>, ) -> Vec2D<i32>
Sourcepub async fn get_image(
&self,
f_cont_locked: Arc<RwLock<FlightComputer>>,
angle: CameraAngle,
) -> Result<(Vec2D<I32F32>, Vec2D<i32>, RgbImage), Box<dyn Error + Send + Sync>>
pub async fn get_image( &self, f_cont_locked: Arc<RwLock<FlightComputer>>, angle: CameraAngle, ) -> Result<(Vec2D<I32F32>, Vec2D<i32>, RgbImage), Box<dyn Error + Send + Sync>>
Performs the HTTP request to retrieve an image from the DRS backend. Then calculates the position and image offset.
Arguments
f_cont_locked: A shared RwLock containing the FlightComputer instance
angle: The current CameraAngle
Returns
A Result containing a tuple with:
- The
Vec2D<I32F32>position where the image was taken - The
Vec2D<i32>offset in the map image buffer - The decoded
RgbImageor an Error
Sourcepub async fn shoot_image_to_map_buffer(
&self,
f_cont_locked: Arc<RwLock<FlightComputer>>,
angle: CameraAngle,
) -> Result<(Vec2D<I32F32>, Vec2D<u32>), Box<dyn Error + Send + Sync>>
pub async fn shoot_image_to_map_buffer( &self, f_cont_locked: Arc<RwLock<FlightComputer>>, angle: CameraAngle, ) -> Result<(Vec2D<I32F32>, Vec2D<u32>), Box<dyn Error + Send + Sync>>
Sourcepub async fn shoot_image_to_zo_buffer(
&self,
f_cont_locked: Arc<RwLock<FlightComputer>>,
angle: CameraAngle,
zoned_objective_map_image: Option<&mut OffsetZonedObjectiveImage>,
) -> Result<Vec2D<I32F32>, Box<dyn Error + Send + Sync>>
pub async fn shoot_image_to_zo_buffer( &self, f_cont_locked: Arc<RwLock<FlightComputer>>, angle: CameraAngle, zoned_objective_map_image: Option<&mut OffsetZonedObjectiveImage>, ) -> Result<Vec2D<I32F32>, Box<dyn Error + Send + Sync>>
Captures an image, processes it, and stores it in a custom zoned objective buffer.
Arguments
f_cont_locked- The lock-protected flight computer.angle- The camera angle and field of view.zoned_objective_map_image: An optional mutable reference to anOffsetZonedObjectiveImage
Returns
The imaging position as Vec2D<I32F32> or an error.
Sourceasync fn update_thumbnail_area_from_fullsize(
&self,
offset: Vec2D<u32>,
size: u32,
)
async fn update_thumbnail_area_from_fullsize( &self, offset: Vec2D<u32>, size: u32, )
Updates the thumbnail area of the map based on the full-size map data.
Arguments
offset- Offset to update.size- Size of the region to update.
Sourcefn decode_png_data(
collected_png: &[u8],
angle: CameraAngle,
) -> Result<RgbImage, Box<dyn Error + Send + Sync>>
fn decode_png_data( collected_png: &[u8], angle: CameraAngle, ) -> Result<RgbImage, Box<dyn Error + Send + Sync>>
Sourcepub(crate) async fn export_and_upload_objective_png(
&self,
objective_id: usize,
offset: Vec2D<u32>,
size: Vec2D<u32>,
export_path: Option<PathBuf>,
zoned_objective_map_image: Option<&OffsetZonedObjectiveImage>,
) -> Result<(), Box<dyn Error>>
pub(crate) async fn export_and_upload_objective_png( &self, objective_id: usize, offset: Vec2D<u32>, size: Vec2D<u32>, export_path: Option<PathBuf>, zoned_objective_map_image: Option<&OffsetZonedObjectiveImage>, ) -> Result<(), Box<dyn Error>>
Exports a specific region of the map as a PNG and uploads it to the server associated with the given objective ID.
Arguments
objective_id- The identifier of the objective to associate the exported PNG with.offset- The offset in the map to start the export.size- The dimensions of the region to export as a PNG.
Returns
A result indicating the success or failure of the operation.
Sourcepub(crate) fn generate_zo_img_path(id: usize) -> PathBuf
pub(crate) fn generate_zo_img_path(id: usize) -> PathBuf
Sourcepub(crate) async fn upload_daily_map_png(&self) -> Result<(), Box<dyn Error>>
pub(crate) async fn upload_daily_map_png(&self) -> Result<(), Box<dyn Error>>
Uploads the daily map snapshot as a PNG to the server.
Returns
A result indicating the success or failure of the operation.
Sourcepub(crate) async fn create_thumb_snapshot(&self) -> Result<(), Box<dyn Error>>
pub(crate) async fn create_thumb_snapshot(&self) -> Result<(), Box<dyn Error>>
Creates and saves a thumbnail snapshot of the map.
Returns
A result indicating the success or failure of the operation.
Sourcepub(crate) async fn export_full_snapshot(&self) -> Result<(), Box<dyn Error>>
pub(crate) async fn export_full_snapshot(&self) -> Result<(), Box<dyn Error>>
Creates and saves a full-size snapshot of the map.
Returns
A result indicating the success or failure of the operation.
Sourcepub(crate) async fn export_thumbnail_png(
&self,
offset: Vec2D<u32>,
angle: CameraAngle,
) -> Result<EncodedImageExtract, Box<dyn Error>>
pub(crate) async fn export_thumbnail_png( &self, offset: Vec2D<u32>, angle: CameraAngle, ) -> Result<EncodedImageExtract, Box<dyn Error>>
Sourcepub(crate) async fn export_full_thumbnail_png(
&self,
) -> Result<EncodedImageExtract, Box<dyn Error>>
pub(crate) async fn export_full_thumbnail_png( &self, ) -> Result<EncodedImageExtract, Box<dyn Error>>
Exports the entire map thumbnail as a PNG.
Returns
A result containing the extracted PNG image data or an error.
Sourcepub(crate) async fn diff_thumb_snapshot(
&self,
) -> Result<EncodedImageExtract, Box<dyn Error>>
pub(crate) async fn diff_thumb_snapshot( &self, ) -> Result<EncodedImageExtract, Box<dyn Error>>
Compares the thumbnail map with its saved snapshot.
Returns
A result containing the difference as an encoded PNG image or an error.
Sourcepub async fn execute_acquisition_cycle(
self: &Arc<Self>,
f_cont_lock: Arc<RwLock<FlightComputer>>,
console_messenger: Arc<ConsoleMessenger>,
__arg3: (DateTime<Utc>, Receiver<PeriodicImagingEndSignal>),
image_max_dt: I32F32,
start_index: usize,
) -> Vec<(isize, isize)>
pub async fn execute_acquisition_cycle( self: &Arc<Self>, f_cont_lock: Arc<RwLock<FlightComputer>>, console_messenger: Arc<ConsoleMessenger>, __arg3: (DateTime<Utc>, Receiver<PeriodicImagingEndSignal>), image_max_dt: I32F32, start_index: usize, ) -> Vec<(isize, isize)>
Executes a series of image acquisitions, processes them, and updates the associated map buffers.
Arguments
f_cont_lock- Lock-protected flight computer controlling the acquisition cycle.console_messenger- Used for sending notifications during processing.(end_time, last_img_kill)- The end time for the cycle and a notify object to terminate the process prematurely.image_max_dt- Maximum allowed interval between consecutive images.lens- The camera angle and field of view.start_index- The starting index for tracking image acquisitions.
Returns
A vector of completed (start, end) time ranges when images were successfully taken.
Sourcepub async fn execute_zo_target_cycle(
self: Arc<Self>,
f_cont_lock: Arc<RwLock<FlightComputer>>,
deadline: DateTime<Utc>,
zoned_objective_image_buffer: &mut Option<OffsetZonedObjectiveImage>,
offset: Vec2D<u32>,
dimensions: Vec2D<u32>,
)
pub async fn execute_zo_target_cycle( self: Arc<Self>, f_cont_lock: Arc<RwLock<FlightComputer>>, deadline: DateTime<Utc>, zoned_objective_image_buffer: &mut Option<OffsetZonedObjectiveImage>, offset: Vec2D<u32>, dimensions: Vec2D<u32>, )
Executes a series of image acquisitions, processes them, and updates an associated zoned objective buffer.
Arguments
f_cont_lock- Lock-protected flight computer controlling the acquisition cycle.deadline- The end time for the cycle.zoned_objective_image_buffer- An optional mutable reference to anOffsetZonedObjectiveImageoffset- The offset of the buffer in the global map buffer.dimensions- The dimensions of the zoned objective.
Sourceasync fn exec_map_capture(
self: &Arc<Self>,
f_cont: &Arc<RwLock<FlightComputer>>,
p_c: &Arc<Mutex<i32>>,
lens: CameraAngle,
) -> (DateTime<Utc>, Option<Vec2D<u32>>)
async fn exec_map_capture( self: &Arc<Self>, f_cont: &Arc<RwLock<FlightComputer>>, p_c: &Arc<Mutex<i32>>, lens: CameraAngle, ) -> (DateTime<Utc>, Option<Vec2D<u32>>)
Captures a single image during mapping operation.
Arguments
f_cont_lock- Lock-protected flight computer controlling the acquisition cycle.p_c- A shared, locked reference to the number of pictures in the current cycle.lens- The desiredCameraAnglefor this picture
Returns
A tuple containing:
- The UTC timestamp when the image was taken
- The
Vec2D<i32>offset in the global map image buffer